@@ -31,6 +31,22 @@
  * Boston, MA  02110-1301, USA.
  *
  *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
+ *
+ *
  * If you didn't download this code from the following link, you should check if
  * you aren't using an obsolete version:
  * http://www.lowagie.com/iText/
@@ -288,7 +304,7 @@ public class AcroFields {
     public void exportAsFdf(FdfWriter writer) {
         for (Iterator it = fields.entrySet().iterator(); it.hasNext();) {
             Map.Entry entry = (Map.Entry)it.next();
-            Item item = (Item)entry.getValue();
+            // ssteward omit: Item item = (Item)entry.getValue();
             String name = (String)entry.getKey();
             // PdfObject v = PdfReader.getPdfObject(((PdfDictionary)item.merged.get(0)).get(PdfName.V));
 			// ssteward: moved this logic to getField, where lastWasString is set;
@@ -1539,7 +1555,7 @@ public class AcroFields {
         }
     }
     
-    private void updateByteRange(PdfPKCS7 pkcs7, PdfDictionary v) {
+    private void updateByteRange(PdfPKCS7 pkcs7, PdfDictionary v) throws IOException {
         PdfArray b = (PdfArray)PdfReader.getPdfObject(v.get(PdfName.BYTERANGE));
         RandomAccessFileOrArray rf = reader.getSafeFile();
         try {
