@@ -30,6 +30,22 @@
  * Boston, MA  02110-1301, USA.
  *
  *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
+ *
+ *
  * If you didn't download this code from the following link, you should check if
  * you aren't using an obsolete version:
  * http://www.lowagie.com/iText/
@@ -108,7 +124,7 @@ public class PdfSignatureAppearance {
     private PdfSigGenericPKCS sigStandard;
     private int range[];
     private RandomAccessFile raf;
-    private int rangePosition = 0;
+    // ssteward omit: private int rangePosition = 0;
     private byte bout[];
     private int boutLen;
     private byte externalDigest[];
@@ -915,7 +931,7 @@ public class PdfSignatureAppearance {
                 try{originalout.close();}catch(Exception e){}
         }
     }
-    
+    /* ssteward omit: 
     private static int indexArray(byte bout[], int position, String search) {
         byte ss[] = PdfEncodings.convertToBytes(search, null);
         while (true) {
@@ -929,7 +945,7 @@ public class PdfSignatureAppearance {
             ++position;
         }
     }
-
+    
     private static int indexFile(RandomAccessFile raf, int position, String search) throws IOException {
         byte ss[] = PdfEncodings.convertToBytes(search, null);
         while (true) {
@@ -947,6 +963,7 @@ public class PdfSignatureAppearance {
             ++position;
         }
     }
+    */
     
     /**
      * Gets the document bytes that are hashable when using external signatures. The general sequence is:
